@@ -407,12 +407,12 @@ impl_ppp_manager_need_secrets (NMPPPManager *manager,
 	 * servers (T-Mobile USA) appear to ask a few times when they actually don't
 	 * even care what you pass back.
 	 */
-	nm_act_request_request_connection_secrets (priv->act_req,
-	                                           setting_name,
-	                                           tries > 1 ? TRUE : FALSE,
-	                                           SECRETS_CALLER_PPP,
-	                                           hint1,
-	                                           hint2);
+	nm_act_request_get_secrets (priv->act_req,
+	                            setting_name,
+	                            tries > 1 ? TRUE : FALSE,
+	                            SECRETS_CALLER_PPP,
+	                            hint1,
+	                            hint2);
 	g_object_set_data (G_OBJECT (connection), PPP_MANAGER_SECRET_TRIES, GUINT_TO_POINTER (++tries));
 	priv->pending_secrets_context = context;
 
@@ -478,42 +478,6 @@ impl_ppp_manager_set_ip4_config (NMPPPManager *manager,
 
 		for (i = 0; i < dns->len; i++)
 			nm_ip4_config_add_nameserver (config, g_array_index (dns, guint, i));
-
-		/* Work around a PPP bug (#1732) which causes many mobile broadband
-		 * providers to return 10.11.12.13 and 10.11.12.14 for the DNS servers.
-		 * Apparently fixed in ppp-2.4.5 but we've had some reports that this is
-		 * not the case.
-		 *
-		 * http://git.ozlabs.org/?p=ppp.git;a=commitdiff_plain;h=2e09ef6886bbf00bc5a9a641110f801e372ffde6
-		 * http://git.ozlabs.org/?p=ppp.git;a=commitdiff_plain;h=f8191bf07df374f119a07910a79217c7618f113e
-		 */
-
-		if (dns->len == 2) {
-			guint32 bad_dns1 = htonl (0x0A0B0C0D);
-			guint32 good_dns1 = htonl (0x04020201);  /* GTE nameserver */
-			guint32 bad_dns2 = htonl (0x0A0B0C0E);
-			guint32 good_dns2 = htonl (0x04020202);  /* GTE nameserver */
-			gboolean found1 = FALSE, found2 = FALSE;
-
-			for (i = 0; i < dns->len; i++) {
-				guint32 ns = nm_ip4_config_get_nameserver (config, i);
-
-				if (ns == bad_dns1)
-					found1 = TRUE;
-				else if (ns == bad_dns2)
-					found2 = TRUE;
-			}
-
-			/* Be somewhat conservative about substitutions; the "bad" nameservers
-			 * could actually be valid in some cases, so only substitute if ppp
-			 * returns *only* the two bad nameservers.
-			 */
-			if (found1 && found2) {
-				nm_ip4_config_reset_nameservers (config);
-				nm_ip4_config_add_nameserver (config, good_dns1);
-				nm_ip4_config_add_nameserver (config, good_dns2);
-			}
-		}
 	}
 
 	val = (GValue *) g_hash_table_lookup (config_hash, NM_PPP_IP4_CONFIG_WINS);
