@@ -22,25 +22,23 @@
  */
 
 #include <config.h>
-#include <gmodule.h>
-#include <glib-object.h>
-#include <glib/gi18n.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <net/ethernet.h>
 #include <netinet/ether.h>
 
+#include <gmodule.h>
+#include <glib-object.h>
+#include <glib/gi18n.h>
+#include <gio/gio.h>
+
+#include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 
 #include <nm-setting-connection.h>
 
-#ifndef NO_GIO
-#include <gio/gio.h>
-#else
-#include <gfilemonitor/gfilemonitor.h>
-#endif
-
 #include "common.h"
 #include "nm-dbus-glib-types.h"
 #include "plugin.h"
@@ -51,6 +49,17 @@
 #include "writer.h"
 #include "utils.h"
 
+#define DBUS_SERVICE_NAME "com.redhat.ifcfgrh1"
+#define DBUS_OBJECT_PATH "/com/redhat/ifcfgrh1"
+
+static gboolean impl_ifcfgrh_get_ifcfg_details (SCPluginIfcfg *plugin,
+                                                const char *in_ifcfg,
+                                                const char **out_uuid,
+                                                const char **out_path,
+                                                GError **error);
+
+#include "nm-ifcfg-rh-glue.h"
+
 static void system_config_interface_init (NMSystemConfigInterface *system_config_interface_class);
 
 static void connection_changed_handler (SCPluginIfcfg *plugin,
@@ -73,9 +82,6 @@ G_DEFINE_TYPE_EXTENDED (SCPluginIfcfg, sc_plugin_ifcfg, G_TYPE_OBJECT, 0,
 
 
 typedef struct {
-	DBusGConnection *g_connection;
-	NMSystemConfigHalManager *hal_mgr;
-
 	GHashTable *connections;
 
 	gulong ih_event_id;
@@ -84,49 +90,17 @@ typedef struct {
 
 	GFileMonitor *monitor;
 	guint monitor_id;
+
+	DBusGConnection *bus;
 } SCPluginIfcfgPrivate;
 
 
-static void
-check_unmanaged (gpointer key, gpointer data, gpointer user_data)
-{
-	GSList **list = (GSList **) user_data;
-	NMIfcfgConnection *connection = NM_IFCFG_CONNECTION (data);
-	const char *udi;
-	GSList *iter;
-
-	if (!nm_ifcfg_connection_get_unmanaged (connection))
-		return;
-
-	udi = nm_ifcfg_connection_get_udi (connection);
-	if (!udi)
-		return;
-
-	/* Just return if the UDI is already in the list */
-	for (iter = *list; iter; iter = g_slist_next (iter)) {
-		if (!strcmp ((char *) iter->data, udi))
-			return;
-	}
-
-	*list = g_slist_prepend (*list, g_strdup (udi));
-}
-
-static GSList *
-get_unmanaged_devices (NMSystemConfigInterface *config)
-{
-	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (config);	
-	GSList *list = NULL;
-
-	g_hash_table_foreach (priv->connections, check_unmanaged, &list);
-	return list;
-}
-
 static void
 connection_unmanaged_changed (NMIfcfgConnection *connection,
                               GParamSpec *pspec,
                               gpointer user_data)
 {
-	g_signal_emit_by_name (SC_PLUGIN_IFCFG (user_data), "unmanaged-devices-changed");
+	g_signal_emit_by_name (SC_PLUGIN_IFCFG (user_data), NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
 }
 
 static void
@@ -153,15 +127,12 @@ read_one_connection (SCPluginIfcfg *plugin, const char *filename)
 
 	PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "parsing %s ... ", filename);
 
-	connection = nm_ifcfg_connection_new (filename, priv->g_connection, priv->hal_mgr, &error, &ignore_error);
+	connection = nm_ifcfg_connection_new (filename, &error, &ignore_error);
 	if (connection) {
-		NMConnection *wrapped;
 		NMSettingConnection *s_con;
 		const char *cid;
 
-		wrapped = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (connection));
-		g_assert (wrapped);
-		s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (wrapped, NM_TYPE_SETTING_CONNECTION));
+		s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_CONNECTION);
 		g_assert (s_con);
 
 		cid = nm_setting_connection_get_id (s_con);
@@ -172,10 +143,10 @@ read_one_connection (SCPluginIfcfg *plugin, const char *filename)
 		                     g_object_ref (connection));
 		PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "    read connection '%s'", cid);
 
-		if (nm_ifcfg_connection_get_unmanaged (connection)) {
+		if (nm_ifcfg_connection_get_unmanaged_spec (connection)) {
 			PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "Ignoring connection '%s' and its "
 			              "device because NM_CONTROLLED was false.", cid);
-			g_signal_emit_by_name (plugin, "unmanaged-devices-changed");
+			g_signal_emit_by_name (plugin, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
 		} else {
 			/* Wait for the connection to become unmanaged once it knows the
 			 * UDI of it's device, if/when the device gets plugged in.
@@ -235,11 +206,10 @@ connection_changed_handler (SCPluginIfcfg *plugin,
                             gboolean *do_remove,
                             gboolean *do_new)
 {
-	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (plugin);
-	NMIfcfgConnection *tmp;
+	NMIfcfgConnection *new;
 	GError *error = NULL;
-	GHashTable *settings;
-	gboolean new_unmanaged, old_unmanaged, ignore_error = FALSE;
+	gboolean ignore_error = FALSE;
+	const char *new_unmanaged = NULL, *old_unmanaged = NULL;
 
 	g_return_if_fail (plugin != NULL);
 	g_return_if_fail (path != NULL);
@@ -249,11 +219,8 @@ connection_changed_handler (SCPluginIfcfg *plugin,
 
 	PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "updating %s", path);
 
-	tmp = (NMIfcfgConnection *) nm_ifcfg_connection_new (path, priv->g_connection,
-	                                                     priv->hal_mgr,
-	                                                     &error,
-	                                                     &ignore_error);
-	if (!tmp) {
+	new = (NMIfcfgConnection *) nm_ifcfg_connection_new (path, &error, &ignore_error);
+	if (!new) {
 		/* errors reading connection; remove it */
 		if (!ignore_error) {
 			PLUGIN_WARN (IFCFG_PLUGIN_NAME, "    error: %s",
@@ -268,8 +235,8 @@ connection_changed_handler (SCPluginIfcfg *plugin,
 
 	/* Successfully read connection changes */
 
-	old_unmanaged = nm_ifcfg_connection_get_unmanaged (NM_IFCFG_CONNECTION (connection));
-	new_unmanaged = nm_ifcfg_connection_get_unmanaged (NM_IFCFG_CONNECTION (tmp));
+	old_unmanaged = nm_ifcfg_connection_get_unmanaged_spec (NM_IFCFG_CONNECTION (connection));
+	new_unmanaged = nm_ifcfg_connection_get_unmanaged_spec (NM_IFCFG_CONNECTION (new));
 
 	if (new_unmanaged) {
 		if (!old_unmanaged) {
@@ -277,16 +244,11 @@ connection_changed_handler (SCPluginIfcfg *plugin,
 			*do_remove = *do_new = TRUE;
 		}
 	} else {
-		NMConnection *old_wrapped, *new_wrapped;
-
-		new_wrapped = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (tmp));
-		old_wrapped = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (connection));
-
-		if (old_unmanaged) {  /* no longer unmanaged */
+		if (old_unmanaged) {  /* now managed */
 			NMSettingConnection *s_con;
 			const char *cid;
 
-			s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (new_wrapped, NM_TYPE_SETTING_CONNECTION));
+			s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (new), NM_TYPE_SETTING_CONNECTION);
 			g_assert (s_con);
 
 			cid = nm_setting_connection_get_id (s_con);
@@ -294,25 +256,23 @@ connection_changed_handler (SCPluginIfcfg *plugin,
 
 			PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "Managing connection '%s' and its "
 			              "device because NM_CONTROLLED was true.", cid);
-			g_signal_emit_by_name (plugin, "connection-added", connection);
+			g_signal_emit_by_name (plugin, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, connection);
 		}
 
-		/* Only update if different */
-		if (!nm_connection_compare (new_wrapped, old_wrapped, NM_SETTING_COMPARE_FLAG_EXACT)) {
-			settings = nm_connection_to_hash (new_wrapped);
-			if (!nm_ifcfg_connection_update (connection, settings, &error)) {
-				PLUGIN_WARN (IFCFG_PLUGIN_NAME, "    error updating: %s",
-				             (error && error->message) ? error->message : "(unknown)");
-				g_clear_error (&error);
-			}
-			g_hash_table_destroy (settings);
+		if (!nm_sysconfig_connection_update (NM_SYSCONFIG_CONNECTION (connection),
+		                                     NM_CONNECTION (new),
+		                                     TRUE,
+		                                     &error)) {
+			PLUGIN_WARN (IFCFG_PLUGIN_NAME, "    error updating: %s",
+			             (error && error->message) ? error->message : "(unknown)");
+			g_clear_error (&error);
 		}
 
 		/* Update unmanaged status */
 		g_object_set (connection, "unmanaged", new_unmanaged, NULL);
-		g_signal_emit_by_name (plugin, "unmanaged-devices-changed");
+		g_signal_emit_by_name (plugin, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
 	}
-	g_object_unref (tmp);
+	g_object_unref (new);
 }
 
 static void
@@ -328,24 +288,24 @@ handle_connection_remove_or_new (SCPluginIfcfg *plugin,
 	g_return_if_fail (path != NULL);
 
 	if (do_remove) {
-		gboolean unmanaged;
+		const char *unmanaged;
 
 		g_return_if_fail (connection != NULL);
 
-		unmanaged = nm_ifcfg_connection_get_unmanaged (connection);
+		unmanaged = nm_ifcfg_connection_get_unmanaged_spec (connection);
 		g_hash_table_remove (priv->connections, path);
-		nm_exported_connection_signal_removed (NM_EXPORTED_CONNECTION (connection));
+		g_signal_emit_by_name (connection, "removed");
 
 		/* Emit unmanaged changes _after_ removing the connection */
 		if (unmanaged)
-			g_signal_emit_by_name (plugin, "unmanaged-devices-changed");
+			g_signal_emit_by_name (plugin, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
 	}
 
 	if (do_new) {
 		connection = read_one_connection (plugin, path);
 		if (connection) {
-			if (!nm_ifcfg_connection_get_unmanaged (NM_IFCFG_CONNECTION (connection)))
-				g_signal_emit_by_name (plugin, "connection-added", connection);
+			if (!nm_ifcfg_connection_get_unmanaged_spec (connection))
+				g_signal_emit_by_name (plugin, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, connection);
 		}
 	}
 }
@@ -415,30 +375,65 @@ setup_ifcfg_monitoring (SCPluginIfcfg *plugin)
 	}
 }
 
-static void
-hash_to_slist (gpointer key, gpointer value, gpointer user_data)
-{
-	NMIfcfgConnection *exported = NM_IFCFG_CONNECTION (value);
-	GSList **list = (GSList **) user_data;
-
-	if (!nm_ifcfg_connection_get_unmanaged (exported))
-		*list = g_slist_prepend (*list, value);
-}
-
 static GSList *
 get_connections (NMSystemConfigInterface *config)
 {
 	SCPluginIfcfg *plugin = SC_PLUGIN_IFCFG (config);
 	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (plugin);
 	GSList *list = NULL;
+	GHashTableIter iter;
+	gpointer value;
+
+	if (!priv->connections) {
+		setup_ifcfg_monitoring (plugin);
+		read_connections (plugin);
+	}
+
+	g_hash_table_iter_init (&iter, priv->connections);
+	while (g_hash_table_iter_next (&iter, NULL, &value)) {
+		NMIfcfgConnection *exported = NM_IFCFG_CONNECTION (value);
+
+		if (!nm_ifcfg_connection_get_unmanaged_spec (exported))
+			list = g_slist_prepend (list, value);
+	}
+
+	return list;
+}
+
+static void
+check_unmanaged (gpointer key, gpointer data, gpointer user_data)
+{
+	GSList **list = (GSList **) user_data;
+	NMIfcfgConnection *connection = NM_IFCFG_CONNECTION (data);
+	const char *unmanaged_spec;
+	GSList *iter;
+
+	unmanaged_spec = nm_ifcfg_connection_get_unmanaged_spec (connection);
+	if (!unmanaged_spec)
+		return;
+
+	/* Just return if the unmanaged spec is already in the list */
+	for (iter = *list; iter; iter = g_slist_next (iter)) {
+		if (!strcmp ((char *) iter->data, unmanaged_spec))
+			return;
+	}
+
+	*list = g_slist_prepend (*list, g_strdup (unmanaged_spec));
+}
+
+static GSList *
+get_unmanaged_specs (NMSystemConfigInterface *config)
+{
+	SCPluginIfcfg *plugin = SC_PLUGIN_IFCFG (config);
+	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (config);
+	GSList *list = NULL;
 
 	if (!priv->connections) {
 		setup_ifcfg_monitoring (plugin);
 		read_connections (plugin);
 	}
 
-	g_hash_table_foreach (priv->connections, hash_to_slist, &list);
-
+	g_hash_table_foreach (priv->connections, check_unmanaged, &list);
 	return list;
 }
 
@@ -526,34 +521,110 @@ sc_network_changed_cb (NMInotifyHelper *ih,
 		g_free (new_hostname);
 }
 
-static void
-init (NMSystemConfigInterface *config, NMSystemConfigHalManager *hal_manager)
+static gboolean
+impl_ifcfgrh_get_ifcfg_details (SCPluginIfcfg *plugin,
+                                const char *in_ifcfg,
+                                const char **out_uuid,
+                                const char **out_path,
+                                GError **error)
 {
-	SCPluginIfcfg *plugin = SC_PLUGIN_IFCFG (config);
 	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (plugin);
+	NMIfcfgConnection *connection;
+	NMSettingConnection *s_con;
+	const char *uuid;
+	const char *path;
 
-	priv->hal_mgr = g_object_ref (hal_manager);
+	if (!g_path_is_absolute (in_ifcfg)) {
+		g_set_error (error, 0, 0, "ifcfg path '%s' is not absolute", in_ifcfg);
+		return FALSE;
+	}
+
+	connection = g_hash_table_lookup (priv->connections, in_ifcfg);
+	if (!connection) {
+		g_set_error (error, 0, 0, "ifcfg file '%s' unknown", in_ifcfg);
+		return FALSE;
+	}
+
+	s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_CONNECTION);
+	if (!s_con) {
+		g_set_error_literal (error, 0, 0, "unable to retrieve the connection setting");
+		return FALSE;
+	}
+
+	uuid = nm_setting_connection_get_uuid (s_con);
+	if (!uuid) {
+		g_set_error_literal (error, 0, 0, "unable to get the UUID");
+		return FALSE;
+	}
+	
+	path = nm_connection_get_path (NM_CONNECTION (connection));
+	if (!path) {
+		g_set_error_literal (error, 0, 0, "unable to get the connection D-Bus path");
+		return FALSE;
+	}
+
+	*out_uuid = g_strdup (uuid);
+	*out_path = g_strdup (path);
+
+	return TRUE;
+}
+
+static void
+init (NMSystemConfigInterface *config)
+{
 }
 
 static void
 sc_plugin_ifcfg_init (SCPluginIfcfg *plugin)
 {
 	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (plugin);
-	GError *error = NULL;
 	NMInotifyHelper *ih;
-
-	priv->g_connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (!priv->g_connection) {
-		PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "    dbus-glib error: %s",
-		              error->message ? error->message : "(unknown)");
-		g_error_free (error);
-	}
+	GError *error = NULL;
+	gboolean success = FALSE;
 
 	ih = nm_inotify_helper_get ();
 	priv->ih_event_id = g_signal_connect (ih, "event", G_CALLBACK (sc_network_changed_cb), plugin);
 	priv->sc_network_wd = nm_inotify_helper_add_watch (ih, SC_NETWORK_FILE);
 
 	priv->hostname = plugin_get_hostname (plugin);
+
+	priv->bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (!priv->bus) {
+		PLUGIN_WARN (IFCFG_PLUGIN_NAME, "Couldn't connect to D-Bus: %s",
+		             error->message);
+		g_clear_error (&error);
+	} else {
+		DBusConnection *tmp;
+		DBusGProxy *proxy;
+		int result;
+
+		tmp = dbus_g_connection_get_connection (priv->bus);
+		dbus_connection_set_exit_on_disconnect (tmp, FALSE);
+
+		proxy = dbus_g_proxy_new_for_name (priv->bus,
+		                                   "org.freedesktop.DBus",
+		                                   "/org/freedesktop/DBus",
+		                                   "org.freedesktop.DBus");
+
+		if (!dbus_g_proxy_call (proxy, "RequestName", &error,
+		                        G_TYPE_STRING, DBUS_SERVICE_NAME,
+		                        G_TYPE_UINT, DBUS_NAME_FLAG_DO_NOT_QUEUE,
+		                        G_TYPE_INVALID,
+		                        G_TYPE_UINT, &result,
+		                        G_TYPE_INVALID)) {
+			PLUGIN_WARN (IFCFG_PLUGIN_NAME, "Couldn't acquire D-Bus service: %s",
+			             error->message);
+			g_clear_error (&error);
+		} else if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
+			PLUGIN_WARN (IFCFG_PLUGIN_NAME, "Couldn't acquire ifcfgrh1 D-Bus service (already taken)");
+		} else
+			success = TRUE;
+	}
+
+	if (!success) {
+		dbus_g_connection_unref (priv->bus);
+		priv->bus = NULL;
+	}
 }
 
 static void
@@ -563,7 +634,10 @@ dispose (GObject *object)
 	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (plugin);
 	NMInotifyHelper *ih;
 
-	g_object_unref (priv->hal_mgr);
+	if (priv->bus) {
+		dbus_g_connection_unref (priv->bus);
+		priv->bus = NULL;
+	}
 
 	ih = nm_inotify_helper_get ();
 
@@ -574,9 +648,6 @@ dispose (GObject *object)
 
 	g_free (priv->hostname);
 
-	if (priv->g_connection)
-		dbus_g_connection_unref (priv->g_connection);
-
 	if (priv->connections)
 		g_hash_table_destroy (priv->connections);
 
@@ -668,6 +739,9 @@ sc_plugin_ifcfg_class_init (SCPluginIfcfgClass *req_class)
 	g_object_class_override_property (object_class,
 	                                  NM_SYSTEM_CONFIG_INTERFACE_PROP_HOSTNAME,
 	                                  NM_SYSTEM_CONFIG_INTERFACE_HOSTNAME);
+
+	dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (req_class),
+									 &dbus_glib_nm_ifcfg_rh_object_info);
 }
 
 static void
@@ -676,7 +750,7 @@ system_config_interface_init (NMSystemConfigInterface *system_config_interface_c
 	/* interface implementation */
 	system_config_interface_class->get_connections = get_connections;
 	system_config_interface_class->add_connection = add_connection;
-	system_config_interface_class->get_unmanaged_devices = get_unmanaged_devices;
+	system_config_interface_class->get_unmanaged_specs = get_unmanaged_specs;
 	system_config_interface_class->init = init;
 }
 
@@ -684,10 +758,19 @@ G_MODULE_EXPORT GObject *
 nm_system_config_factory (void)
 {
 	static SCPluginIfcfg *singleton = NULL;
+	SCPluginIfcfgPrivate *priv;
 
-	if (!singleton)
+	if (!singleton) {
 		singleton = SC_PLUGIN_IFCFG (g_object_new (SC_TYPE_PLUGIN_IFCFG, NULL));
-	else
+		if (singleton) {
+			priv = SC_PLUGIN_IFCFG_GET_PRIVATE (singleton);
+			if (priv->bus)
+				dbus_g_connection_register_g_object (priv->bus,
+				                                     DBUS_OBJECT_PATH,
+				                                     G_OBJECT (singleton));
+			PLUGIN_PRINT (IFCFG_PLUGIN_NAME, "Acquired D-Bus service %s", DBUS_SERVICE_NAME);
+		}
+	} else
 		g_object_ref (singleton);
 
 	return G_OBJECT (singleton);
